__author__ = 'maartenbreddels'
import string
import astropy.io.votable.ucd
import astropy.units
import logging
from vaex.ui.qt import *
# from astropy.utils import data
import astropy.utils.data
# astropy.utils.data
import numpy as np

logger = logging.getLogger("vaex.ui.completer")

# based on http://stackoverflow.com/a/26065682


class Completer(QtGui.QCompleter):
    def __init__(self, line_edit, allowed_chars=string.ascii_letters + string.digits + "_", separators=None, match_contains=True, match_case=False):
        QtGui.QCompleter.__init__(self, [], line_edit)
        self.line_edit = line_edit
        self.match_contains = match_contains
        self.match_case = match_case
        self.allowed_chars = allowed_chars
        self.separators = separators
        self.line_edit.cursorPositionChanged.connect(self.onCursorPositionChanged)
        self.last_editing_cursor_position = None
        self.activated.connect(self.onActivated)
        self.setWrapAround(False)
        self.setCompletionMode(QtGui.QCompleter.UnfilteredPopupCompletion)
        self.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
        # self.setCompletionMode(QtGui.QCompleter.InlineCompletion)

        self.model = QtGui.QStringListModel([], self)
        self.setModel(self.model)

    def pathFromIndex(self, index):
        # return QtGui.QCompleter.pathFromIndex(self, index)
        suggested_word = QtGui.QCompleter.pathFromIndex(self, index)

        full_text = self.line_edit.text()

        index = self.line_edit.cursorPosition()
        left, right = self.find_word_bounds(full_text, index, self.allowed_chars)

        suggested_text = full_text[:left] + suggested_word + full_text[right:]
        new_cursor_pos = len(full_text[:left] + suggested_word)

        logger.debug("cursor should go to: %d", new_cursor_pos)

        def fixcursor():
            logger.debug("cursor set to: %d", new_cursor_pos)
            self.line_edit.setCursorPosition(new_cursor_pos)
        # print(("pathFromIndex", index, repr(full_text), repr(suggested_text), repr(suggested_text), self.last_editing_cursor_position))

        # after the text is set by completer, the cursor is set to the end of the lineedit, we correct is by fixcursor to set it at
        # the end of the word
        QtCore.QTimer.singleShot(0, fixcursor)
        return suggested_text

    def splitPath(self, path):
        index = self.line_edit.cursorPosition()
        left, right = self.find_word_bounds(path, index, self.allowed_chars)
        part = path[left:right]

        result = QtGui.QCompleter.splitPath(self, path)
        # print "splitPath", path, result, part

        suggestions = self.get_suggestions(part, path[:left], path[right:])
        self.suggestions = suggestions
        self.model.setStringList(suggestions)
        # self.model = QtGui.QStringListModel(suggestions, self.parent())
        # self.setModel(self.model)
        # QtCore.QTimer.singleShot(0, lambda: self.setModel(self.model));
        self.parts = [part]
        return self.parts

    def onActivated(self, text):
        pass  # print "activated", text

    def onCursorPositionChanged(self, old_pos, new_pos):
        # print "cursor", old_pos, new_pos
        # this trick didn't work, as suggested by the SO anwser
        self.last_editing_cursor_position = None if old_pos == new_pos else new_pos

    def word_boundary_char(self, char):
        return (self.allowed_chars is not None and char not in self.allowed_chars) or\
            (self.separators is not None and char in self.separators)

    def find_word_bounds(self, text, index, allowed_chars):
        right = left = index
        done = False
        while not done:
            if left == 0:
                done = True
            elif not self.word_boundary_char(text[left - 1]):
                left -= 1
            else:
                done = True
        done = False
        while not done:
            if right == len(text):
                done = True
            elif not self.word_boundary_char(text[right]):
                right += 1
            else:
                done = True
        return left, right

    def get_word_list(self, word, text_left, text_right):
        return "aap aardappel schaap koe blaat".split()

    def get_suggestions(self, typed_word, text_left, text_right):
        def case(word):
            return word if self.match_case else word.lower()

        suggestions = []
        if typed_word:
            word_list = self.get_word_list(typed_word, text_left, text_right)
            for word in word_list:

                if (self.match_contains and case(typed_word) in case(word)) or (not self.match_contains and case(word).startswith(case(typed_word))):
                    suggestions.append(word)
        return suggestions


ucd_words = astropy.io.votable.ucd.UCDWords()
primary_list = list(sorted(ucd_words._primary))
secondary_list = list(sorted(ucd_words._secondary))


class UCDCompleter(Completer):
    """
    UCDs have primary words (that come first), and secondary, that come after the second
    UCD words are seperated by a ; char
    """

    def __init__(self, line_edit):
        Completer.__init__(self, line_edit, allowed_chars=None, separators=";")

    def get_word_list(self, word, text_left, text_right):
        if text_left.strip():
            return secondary_list
        else:
            return primary_list

    def get_suggestions(self, typed_word, text_left, text_right):
        typed_word = typed_word.lower()
        if text_left.strip():
            word_list = secondary_list
        else:
            word_list = primary_list
        descriptions = {key: desc.lower() for key, desc in ucd_words._descriptions.items() if key in word_list}
        suggestions = []
        if typed_word:
            for word in word_list:
                # if any([typed_word in word for word in word_list]) or any([typed_word in desc for desc in descriptions]):
                # print(typed_word)
                # print(list([typed_word in desc for desc in descriptions]))
                # print(any([typed_word in desc for desc in descriptions]))
                # print([desc for desc in descriptions if typed_word in desc])
                if (typed_word in word) or typed_word in descriptions[word]:
                    suggestions.append(ucd_words._capitalization[word])
        return suggestions


class IdentifierCompleter(Completer):
    """Completes variables and functions"""

    def __init__(self, line_edit, variables=[]):
        self.variables = variables
        Completer.__init__(self, line_edit)

    def get_word_list(self, word, text_left, text_right):
        return self.variables


unit_list = [name for name, unit in vars(astropy.units).items() if isinstance(unit, astropy.units.UnitBase)]


class UnitCompleter(Completer):
    """Completes units found in astropy"""

    def __init__(self, line_edit, unit_list=unit_list):
        self.unit_list = unit_list
        Completer.__init__(self, line_edit, match_contains=False)

    def get_word_list(self, word, text_left, text_right):
        return self.unit_list


# based on from https://gist.github.com/Riateche/5984815
class UCDDelegate(QtGui.QItemDelegate):
    def __init__(self, parent):
        QtGui.QItemDelegate.__init__(self, parent)

    def createEditor(self, parent, option, index):
        editor = QtGui.QLineEdit(parent)
        completer = vaex.ui.completer.UCDCompleter(editor)
        editor.setCompleter(completer)
        # self.connect(combo, QtCore.SIGNAL("currentIndexChanged(int)"), self, QtCore.SLOT("currentIndexChanged()"))
        return editor

    def setEditorData(self, editor, index):
        editor.blockSignals(True)
        # editor.setCurrentIndex(int(index.model().data(index)))
        editor.setText(index.model().data(index))
        editor.blockSignals(False)

    def setModelData(self, editor, model, index):
        model.setData(index, editor.text())

    def currentIndexChanged(self):
        self.commitData.emit(self.sender())


class UnitDelegate(QtGui.QStyledItemDelegate):
    def __init__(self, parent):
        QtGui.QStyledItemDelegate.__init__(self, parent)
        self.lastEditor = None

    def createEditor(self, parent, option, index):
        self.lastEditor = editor = QtGui.QLineEdit(parent)
        self.completer = vaex.ui.completer.UnitCompleter(editor)
        editor.setCompleter(self.completer)
        # self.connect(combo, QtCore.SIGNAL("currentIndexChanged(int)"), self, QtCore.SLOT("currentIndexChanged()"))

        return editor

    def setEditorData(self, editor, index):
        editor.blockSignals(True)
        # editor.setCurrentIndex(int(index.model().data(index)))
        editor.setText(index.model().data(index))
        editor.blockSignals(False)

    def setModelData(self, editor, model, index):
        model.setData(index, editor.text())

    def currentIndexChanged(self):
        self.commitData.emit(self.sender())


vars = dir(np)
# vars.append("hoeba")


class LineCompleter(QtGui.QLineEdit):
    def __init__(self, parent):
        QtGui.QLineEdit.__init__(self, parent)
        # completer = UCDCompleter(self)
        # assert completer.word_boundary_char(";")
        completer = UnitCompleter(self)
        self.setCompleter(completer)


class ExpressionCombobox(QtGui.QComboBox):
    def __init__(self, parent, dataset, variables=False):
        """

        :param parent:
        :param Dataset dataset:
        """
        QtGui.QComboBox.__init__(self, parent)
        self.identifiers = []
        self.columns = dataset.get_column_names(virtual=True)
        self.identifiers.extend(vaex.dataset.expression_namespace.keys())
        self.identifiers.extend(self.columns)
        if variables:
            self.identifiers.extend(list(dataset.variables.keys()))
        self.addItems([""] + self.columns)
        # = list(dataset.variables.keys()) + list(vaex.dataset.expression_namespace.keys())
        # self.addItems(list(dataset.variables.keys()))
        # else:
        # self.identifiers = list(self.columns) + list(vaex.dataset.expression_namespace.keys())
        self.setEditable(True)
        lineEdit = self.lineEdit()
        self.completer = IdentifierCompleter(lineEdit, self.identifiers)
        lineEdit.setCompleter(self.completer)


if __name__ == "__main__":
    app = QtGui.QApplication([])
    dialog = QtGui.QDialog()
    dialog.resize(400, 100)
    if 1:
        lineEdit = LineCompleter(dialog)
        # combo = QtGui.QComboBox(dialog)
        # combo.setEditable(True)
        # combo.addItems("hoeba blaat schaap".split())
        # lineEdit = combo.lineEdit()
        # completer = MathCompleter(lineEdit, vars)
        completer = UnitCompleter(lineEdit)
        lineEdit.setCompleter(completer)
    else:
        dataset = vaex.open(sys.argv[1])
        box = ExpressionCombobox(dialog, dataset)
    dialog.show()
    dialog.raise_()
    dialog.exec_()
    # app.exec_()



import functools

import matplotlib.widgets

import vaex.ui.plugin
from vaex.ui import undo
from vaex.ui.qt import *
from vaex.ui.icons import iconfile
import logging
import vaex.ui.undo as undo


logger = logging.getLogger("plugin.zoom")

@vaex.ui.plugin.pluginclass
class ZoomPlugin(vaex.ui.plugin.PluginPlot):
	name = "zoom"
	def __init__(self, dialog):
		super(ZoomPlugin, self).__init__(dialog)
		dialog.plug_toolbar(self.plug_toolbar, 1.2)
		
		
	def plug_toolbar(self):
		logger.info("adding zoom plugin")
		self.dialog.menu_mode.addSeparator()
		self.action_zoom_rect = QtGui.QAction(QtGui.QIcon(iconfile('zoom')), '&Zoom to rect', self.dialog)
		self.action_zoom_rect.setShortcut("Ctrl+Alt+Z")
		self.dialog.menu_mode.addAction(self.action_zoom_rect)

		self.action_zoom_x = QtGui.QAction(QtGui.QIcon(iconfile('zoom_x')), '&Zoom x', self.dialog)
		self.action_zoom_y = QtGui.QAction(QtGui.QIcon(iconfile('zoom_y')), '&Zoom y', self.dialog)
		self.action_zoom = QtGui.QAction(QtGui.QIcon(iconfile('zoom')), '&Zoom(you should not read this)', self.dialog)

		self.action_zoom_x.setShortcut("Ctrl+Alt+X")
		self.action_zoom_y.setShortcut("Ctrl+Alt+Y")
		self.dialog.menu_mode.addAction(self.action_zoom_x)
		self.dialog.menu_mode.addAction(self.action_zoom_y)


		self.dialog.menu_mode.addSeparator()
		self.action_zoom_out = QtGui.QAction(QtGui.QIcon(iconfile('zoom_out')), '&Zoom out', self.dialog)
		self.action_zoom_in = QtGui.QAction(QtGui.QIcon(iconfile('zoom_in')), '&Zoom in', self.dialog)
		self.action_zoom_fit = QtGui.QAction(QtGui.QIcon(iconfile('arrow_out')), '&Reset view', self.dialog)
		#self.action_zoom_use = QtGui.QAction(QtGui.QIcon(iconfile('chart_bar')), '&Use zoom area', self.dialog)
		self.action_zoom_out.setShortcut("Ctrl+Alt+-")
		self.action_zoom_in.setShortcut("Ctrl+Alt++")
		self.action_zoom_fit.setShortcut("Ctrl+Alt+0")
		self.dialog.menu_mode.addAction(self.action_zoom_out)
		self.dialog.menu_mode.addAction(self.action_zoom_in)
		self.dialog.menu_mode.addAction(self.action_zoom_fit)



		self.dialog.action_group_main.addAction(self.action_zoom_rect)
		self.dialog.action_group_main.addAction(self.action_zoom_x)
		self.dialog.action_group_main.addAction(self.action_zoom_y)

		#self.dialog.toolbar.addAction(self.action_zoom_out)
		#self.dialog.add_shortcut(self.action_zoom_in,"+")
		#self.dialog.add_shortcut(self.action_zoom_out,"-")
		
		#self.dialog.add_shortcut(self.action_zoom_rect,"Z")
		#self.dialog.add_shortcut(self.action_zoom_x,"Alt+X")
		#self.dialog.add_shortcut(self.action_zoom_y,"Alt+Y")
		#self.dialog.add_shortcut(self.action_zoom_fit, "0")
		
		self.dialog.toolbar.addAction(self.action_zoom)
		self.zoom_menu = QtGui.QMenu()
		self.action_zoom.setMenu(self.zoom_menu)
		self.zoom_menu.addAction(self.action_zoom_rect)
		self.zoom_menu.addAction(self.action_zoom_x)
		self.zoom_menu.addAction(self.action_zoom_y)
		if self.dialog.dimensions == 1:
			self.lastActionZoom = self.action_zoom_x # this makes more sense for histograms as default
		else:
			self.lastActionZoom = self.action_zoom_rect
		
		self.dialog.toolbar.addSeparator()
		#self.dialog.toolbar.addAction(self.action_zoom_out)
		self.dialog.toolbar.addAction(self.action_zoom_fit)

		self.action_zoom.triggered.connect(self.onActionZoom)
		self.action_zoom_out.triggered.connect(self.onZoomOut)
		self.action_zoom_in.triggered.connect(self.onZoomIn)
		self.action_zoom_fit.triggered.connect(self.onZoomFit)
		#self.action_zoom_use.triggered.connect(self.onZoomUse)
		
		
		self.action_zoom.setCheckable(True)
		self.action_zoom_rect.setCheckable(True)
		self.action_zoom_x.setCheckable(True)
		self.action_zoom_y.setCheckable(True)
		
		
	def setMode(self, action):
		useblit = True
		axes_list = self.dialog.getAxesList()
		if action == self.action_zoom_x:
			print("zoom x")
			self.lastActionZoom = self.action_zoom_x
			self.dialog.currentModes = [matplotlib.widgets.SpanSelector(axes, functools.partial(self.onZoomX, axes=axes), 'horizontal', useblit=useblit) for axes in axes_list] #, rectprops={"color":"blue"})
			if useblit:
				self.dialog.canvas.draw() # buggy otherwise
		if action == self.action_zoom_y:
			self.lastActionZoom = self.action_zoom_y
			self.dialog.currentModes = [matplotlib.widgets.SpanSelector(axes, functools.partial(self.onZoomY, axes=axes), 'vertical', useblit=useblit)  for axes in axes_list] #, rectprops={"color":"blue"})
			if useblit:
				self.dialog.canvas.draw() # buggy otherwise
		if action == self.action_zoom_rect:
			print("zoom rect")
			self.lastActionZoom = self.action_zoom_rect
			self.dialog.currentModes = [matplotlib.widgets.RectangleSelector(axes, functools.partial(self.onZoomRect, axes=axes), useblit=useblit) for axes in axes_list] #, rectprops={"color":"blue"})
			if useblit:
				self.dialog.canvas.draw() # buggy otherwise

		

	def onZoomIn(self, *args):
		axes = self.getAxesList()[0] # TODO: handle propery multiple axes
		self.dialog.zoom(0.5, axes)
		self.dialog.queue_history_change("zoom in")
		
	def onZoomOut(self):
		axes = self.dialog.getAxesList()[0] # TODO: handle propery multiple axes
		self.dialog.zoom(2., axes)
		self.dialog.queue_history_change("zoom out")
		

	def onActionZoom(self):
		print("onactionzoom")
		self.lastActionZoom.setChecked(True)
		self.dialog.setMode(self.lastActionZoom)
		self.syncToolbar()
		
	def onZoomFit(self, *args):
		#for i in range(self.dimensions):
		#	self.dialog.ranges[i] = None
		#	self.dialog.state.ranges_viewport[i] = None
		#	self.range_level = None
		if 0:
			for axisIndex in range(self.dimensions):
				linkButton = self.linkButtons[axisIndex]
				link = linkButton.link
				if link:
					logger.debug("sending link messages")
					link.sendRanges(self.dialog.ranges[axisIndex], linkButton)
					link.sendRangesShow(self.dialog.state.ranges_viewport[axisIndex], linkButton)
				
		
		action = undo.ActionZoom(self.dialog.undoManager, "zoom to fit", self.dialog.set_ranges,
		                         list(range(self.dialog.dimensions)),
		                         self.dialog.state.ranges_viewport, self.dialog.state.range_level_show,
						   		 list(range(self.dialog.dimensions)),
							     ranges_viewport=[None] * self.dialog.dimensions, range_level_show=None)
		#for layer in dialog.layers:
		#	layer.range_level = None # reset these... is this the right place?
		action.do()
		self.dialog.checkUndoRedo()
		self.dialog.queue_history_change("zoom to fit")
		
		if 0:
			linked_buttons = [button for button in self.linkButtons if button.link is not None]
			links = [button.link for button in linked_buttons]
			if len(linked_buttons) > 0:
				logger.debug("sending compute message")
				vaex.dataset.Link.sendCompute(links, linked_buttons)
			#linked_buttons[0].sendCompute(blacklist)
		#if linkButtonLast: # only send once
		#	link = linkButtonLast.link
		#	logger.debug("sending compute message")
		#	link.sendCompute(linkButton)

		#self.compute()
		#self.dataset.executor.execute()
		
	def onZoomUse(self, *args):
		# TODO: when this will be an option again, implement this as action
		# TODO: will we ever use this again? auto updates are much better
		for i in range(self.dimensions):
			self.dialog.ranges[i] = self.dialog.state.ranges_viewport[i]
		self.range_level = None
		for axisIndex in range(self.dimensions):
			linkButton = self.linkButtons[axisIndex]
			link = linkButton.link
			if link:
				logger.debug("sending link messages")
				link.sendRanges(self.dialog.ranges[axisIndex], linkButton)
				#link.sendRangesShow(self.dialog.state.ranges_viewport[axisIndex], linkButton)
		linked_buttons = [button for button in self.linkButtons if button.link is not None]
		links = [button.link for button in linked_buttons]
		if len(linked_buttons) > 0:
			logger.debug("sending compute message")
			vaex.dataset.Link.sendCompute(links, linked_buttons)
		self.compute()
		self.dataset.executor.execute()
		
	def onZoomX(self, xmin, xmax, axes):
		
		axisIndex = axes.xaxis_index
		#self.dialog.state.ranges_viewport[axisIndex] = xmin, xmax
		# move the link code to the set ranges
		if 0:
			linkButton = self.linkButtons[axisIndex]
			link = linkButton.link
			if link:
				logger.debug("sending link messages")
				link.sendRangesShow(self.dialog.state.ranges_viewport[axisIndex], linkButton)
				link.sendPlot(linkButton)
		action = undo.ActionZoom(self.dialog.undoManager, "zoom x [%f,%f]" % (xmin, xmax),
						   self.dialog.set_ranges, list(range(self.dialog.dimensions)),
						   self.dialog.state.ranges_viewport,self.dialog.state.range_level_show, [axisIndex], ranges_viewport=[[xmin, xmax]])
		action.do()
		self.dialog.checkUndoRedo()
		self.dialog.queue_history_change("zoom x")

	def onZoomY(self, ymin, ymax, axes):
		if len(self.dialog.state.ranges_viewport) == 1: # if 1d, y refers to range_level
			#self.range_level = ymin, ymax
			action = undo.ActionZoom(self.dialog.undoManager, "change level [%f,%f]" % (ymin, ymax), self.dialog.set_ranges, list(range(self.dialog.dimensions)),
							self.dialog.state.ranges_viewport, self.dialog.state.range_level_show, [], range_level_show=[ymin, ymax])
		else:
			#self.dialog.state.ranges_viewport[axes.yaxis_index] = ymin, ymax
			action = undo.ActionZoom(self.dialog.undoManager, "zoom y [%f,%f]" % (ymin, ymax), self.dialog.set_ranges, list(range(self.dialog.dimensions)),
							self.dialog.state.ranges_viewport, self.dialog.state.range_level_show, [axes.yaxis_index], ranges_viewport=[[ymin, ymax]])
			
		action.do()
		self.dialog.checkUndoRedo()
		self.dialog.queue_history_change("zoom y")

	def onZoomRect(self, eclick, erelease, axes):
		x1, y1 = (eclick.xdata, eclick.ydata)
		x2, y2 = (erelease.xdata, erelease.ydata)
		x = [x1, x2]
		y = [y1, y2]

		range_level = None
		ranges_show = []
		ranges = []
		axis_indices = []
		xmin_show, xmax_show = min(x), max(x)
		ymin_show, ymax_show = min(y), max(y)
		if self.dialog.state.ranges_viewport[0][0] > self.dialog.state.ranges_viewport[0][1]:
			xmin_show, xmax_show = xmax_show, xmin_show
		if len(self.dialog.state.ranges_viewport) == 1 and self.dialog.state.range_level_show[0] > self.dialog.state.range_level_show[1]:
			ymin_show, ymax_show = ymax_show, ymin_show
		elif self.dialog.state.ranges_viewport[1][0] > self.dialog.state.ranges_viewport[1][1]:
			ymin_show, ymax_show = ymax_show, ymin_show

		#self.dialog.state.ranges_viewport[axes.xaxis_index] = xmin_show, xmax_show
		axis_indices.append(axes.xaxis_index)
		ranges_show.append([xmin_show, xmax_show])
		if len(self.dialog.state.ranges_viewport) == 1: # if 1d, y refers to range_level
			#self.range_level = ymin_show, ymax_show
			range_level =  ymin_show, ymax_show
			logger.debug("range refers to level: %r" % (self.dialog.state.range_level_show,))
		else:
			#self.dialog.state.ranges_viewport[axes.yaxis_index] = ymin_show, ymax_show
			axis_indices.append(axes.yaxis_index)
			ranges_show.append([ymin_show, ymax_show])
			
			
		def delayed_zoom():
			action = undo.ActionZoom(self.dialog.undoManager, "zoom to rect", self.dialog.set_ranges, list(range(self.dialog.dimensions)),
							self.dialog.state.ranges_viewport,
							self.dialog.state.range_level_show, axis_indices, ranges_viewport=ranges_show, range_level_show=range_level)
			action.do()
			self.dialog.checkUndoRedo()
		#self.dialog.queue_update(delayed_zoom, delay=300)
		delayed_zoom()
		self.dialog.queue_history_change("zoom to rectangle")

		if 1:
			#self.dialog.state.ranges_viewport = list(ranges_show)
			self.dialog.state.ranges_viewport[axes.xaxis_index] = list(ranges_show[0])
			if self.dialog.dimensions == 2:
				self.dialog.state.ranges_viewport[axes.yaxis_index] = list(ranges_show[1])
				self.dialog.check_aspect(1)
				axes.set_xlim(self.dialog.state.ranges_viewport[0])
				axes.set_ylim(self.dialog.state.ranges_viewport[1])
			if self.dialog.dimensions == 1:
				self.dialog.state.range_level_show = range_level
				axes.set_xlim(self.dialog.state.ranges_viewport[0])
				axes.set_ylim(self.dialog.state.range_level_show)
			self.dialog.queue_redraw()
			
		if 0:

			for axisIndex in range(self.dimensions):
				linkButton = self.linkButtons[axisIndex]
				link = linkButton.link
				if link:
					logger.debug("sending link messages")
					link.sendRangesShow(self.dialog.state.ranges_viewport[axisIndex], linkButton)
					link.sendPlot(linkButton)
			
			#self.axes.set_xlim(self.xmin_show, self.xmax_show)
			#self.axes.set_ylim(self.ymin_show, self.ymax_show)
			#self.canvas.draw()
			action = undo.ActionZoom(self.undoManager, "zoom to rect", self.set_ranges, list(range(self.dimensions)), self.dialog.ranges, self.dialog.state.ranges_viewport,  self.range_level, axis_indices, ranges_viewport=ranges_show, range_level=range_level)
			action.do()
			self.checkUndoRedo()

			if 0:
				if self.autoRecalculate():
					for i in range(self.dimensions):
						self.dialog.ranges[i] = self.dialog.state.ranges_viewport[i]
						self.range_level = None
					self.compute()
					self.dataset.executor.execute()
				else:
					self.plot()
		

	def syncToolbar(self):
		for action in [self.action_zoom]:
			logger.debug("sync action: %r" % action.text())
			subactions = action.menu().actions()
			subaction_selected = [subaction for subaction in subactions if subaction.isChecked()]
			#if len(subaction_selected) > 0:
			#	action.setText(subaction_selected[0].text())
			#	action.setIcon(subaction_selected[0].icon())
			logger.debug(" subaction_selected: %r" % subaction_selected)
			logger.debug(" action was selected?: %r" % action.isChecked())
			action.setChecked(len(subaction_selected) > 0)
			logger.debug(" action  is selected?: %r" % action.isChecked())
		#logger.debug("last select action: %r" % self.lastActionSelect.text())
		logger.debug("last zoom action: %r" % self.lastActionZoom.text())
		#self.action_select.setText(self.lastActionSelect.text())
		#self.action_select.setIcon(self.lastActionSelect.icon())
		self.action_zoom.setText(self.lastActionZoom.text())
		self.action_zoom.setIcon(self.lastActionZoom.icon())




# -*- coding: utf-8 -*-
import matplotlib.cm
import numpy as np
from vaex.ui.qt import *


cols = []
for x in np.linspace(0, 1, 256):
    rcol = 0.237 - 2.13 * x + 26.92 * x**2 - 65.5 * x**3 + 63.5 * x**4 - 22.36 * x**5
    gcol = ((0.572 + 1.524 * x - 1.811 * x**2) / (1 - 0.291 * x + 0.1574 * x**2))**2
    bcol = 1 / (1.579 - 4.03 * x + 12.92 * x**2 - 31.4 * x**3 + 48.6 * x**4 - 23.36 * x**5)
    cols.append((rcol, gcol, bcol))

name = 'PaulT_plusmin'
cm_plusmin = matplotlib.colors.LinearSegmentedColormap.from_list(name, cols)
matplotlib.cm.register_cmap(name=name, cmap=cm_plusmin)

colormaps = []
colormaps_map = {}
cmaps = [	('Extra', ['PaulT_plusmin']),
          ('Sequential', ['binary', 'Blues', 'BuGn', 'BuPu', 'gist_yarg',
                          'GnBu', 'Greens', 'Greys', 'Oranges', 'OrRd',
                          'PuBu', 'PuBuGn', 'PuRd', 'Purples', 'RdPu',
                          'Reds', 'YlGn', 'YlGnBu', 'YlOrBr', 'YlOrRd']),
          ('Sequential (2)', ['afmhot', 'autumn', 'bone', 'cool', 'copper',
                              'gist_gray', 'gist_heat', 'gray', 'hot', 'pink',
                                                                'spring', 'summer', 'winter']),
          ('Diverging', ['BrBG', 'bwr', 'coolwarm', 'PiYG', 'PRGn', 'PuOr',
                         'RdBu', 'RdGy', 'RdYlBu', 'RdYlGn', 'seismic']),
          ('Qualitative', ['Accent', 'Dark2', 'hsv', 'Paired', 'Pastel1',
                           'Pastel2', 'Set1', 'Set2', 'Set3', 'Spectral']),
          ('Miscellaneous', ['gist_earth', 'gist_ncar', 'gist_rainbow',
                             'gist_stern', 'jet', 'brg', 'CMRmap', 'cubehelix',
                             'gnuplot', 'gnuplot2', 'ocean', 'rainbow',
                             'terrain', 'flag', 'prism'])]
for cmap_category, cmap_list in cmaps:
    for colormap_name in cmap_list:
        colormaps.append(colormap_name)


def colormap_to_QImage(colormap, width, height):
    mapping = matplotlib.cm.ScalarMappable(cmap=colormap)
    x = np.arange(width) / (width + 0.)
    x = np.vstack([x] * height)

    rgba = mapping.to_rgba(x, bytes=True)
    # for some reason rgb need to be swapped
    r, g, b = rgba[:, :, 0] * 1., rgba[:, :, 1] * 1., rgba[:, :, 2] * 1.
    rgba[:, :, 0] = b
    rgba[:, :, 1] = g
    rgba[:, :, 2] = r
    stringdata = rgba.tostring()
    image = QtGui.QImage(stringdata, width, height, width * 4, QtGui.QImage.Format_RGB32)
    return image, stringdata


# colormaps = []
colormap_pixmap = {}
colormaps_processed = False
refs = []


def process_colormaps():
    global colormaps_processed
    if colormaps_processed:
        return
    colormaps_processed = True
    for colormap_name in colormaps:
        # colormaps.append(colormap_name)
        Nx, Ny = 32, 16
        image, stringdata = colormap_to_QImage(colormap_name, Nx, Ny)
        refs.append((image, stringdata))
        pixmap = QtGui.QPixmap(32 * 2, 32)
        pixmap.convertFromImage(image)
        colormap_pixmap[colormap_name] = pixmap




order_toolbar_navigate = 1.

def pluginclass(cls):

	cls.__bases__[0].registry.add(cls)
	return cls

def pluginbase(cls):
	if not hasattr(cls, 'registry'):
		cls.registry = set()
	return cls


class RegisterPlugins(type):
	def __init__(cls, name, bases, nmspc):
		super(RegisterPlugins, cls).__init__(name, bases, nmspc)
		if not hasattr(cls, 'registry'):
			cls.registry = set()
		cls.registry.add(cls)
		cls.registry -= set(bases) # Remove base classes

	def __iter__(cls):
		return iter(cls.registry)

	def __str__(cls):
		if cls in cls.registry:
			return cls.__name__
		return cls.__name__ + ": " + ", ".join([sc.__name__ for sc in cls])


@pluginbase
class PluginPlot(object):
	#__metaclass__ = RegisterPlugins
	def __init__(self, dialog):
		self.dialog = dialog

	def clean_up(self):
		pass

	def apply_options(self, options):
		pass

	def get_options(self):
		pass

	def use_layer(self, layer):
		pass

	@staticmethod
	def useon(dialog_class):
		return True

	def syncToolbar(self):
		pass

	def setMode(self, action):
		pass


@pluginbase
class PluginLayer(object):
	#__metaclass__ = RegisterPlugins
	def __init__(self, parent, layer):
		self.parent = parent
		self.layer = layer
		self.dataset = layer.dataset

	def clean_up(self):
		pass
		
	@staticmethod
	def useon(layer_class):
		return True
	
	def get_options(self):
		return {}

	def apply_options(self, options):
		pass

	def start_animation(self, widget):
		pass

	def stop_animation(self):
		pass

@pluginbase
class PluginDataset(object):
	#__metaclass__ = RegisterPlugins
	def __init__(self, dataset, widget):
		self.dataset = dataset
		self.widget = widget

	@staticmethod
	def useon(dataset):
		return True
	
